function [H, sigma2] = dcv_baye(X, v, LFilt, Nit, H_0, sigma2_0);

%dcv_baye  Bayesian analysis of the blind convolution model with the Gibbs
%	   sampler using a noninformative prior.
%Use:	[H,sigma2] = dcv_baye(X,v,LFilt,Nit[,H_0,sigma2_0]). Returns the
%	simulated quantities corresponding to each iteration, ie.
%       H(:,n) and sigma2(n) for n = 1 (initialization) to n = Nit+1 (last
%       iteration). Use a standard "noninformative" prior: uniform on
%	(H, log(sigma_2)). If the initial values (H_0 and sigma2_0)
%       are left unspecified, arbitrary ones are used.
%
%	NB: For the blind convolution model, the posterior is indeed proper
%	under standard conditions.
%
%       Known-bug: Error in simulation of sigma2 (see note below).

% DCV Toolbox,  $Revision: 1.2 $        
% Olivier Capp, 26/12/97 - $Date: 2004/11/19 10:55:37 $version 1.1
% ENST / CNRS, Paris

% Needed functions
if (~((exist('randgamm') == 2) | (exist('randgamm') == 3)))
  error('Function randgamm is missing.');
end;

% Constants
NECHO = 50;

% Input args.
error(nargchk(4, 6, nargin));
if (nargin < 6)
  RANDOM_INIT = 1;
else
  RANDOM_INIT = 0;
end;
% Set parameters for the DCV functions
[T, Nc] = size(X);
if (Nc ~= 1)
  error('Observations must be specified as a column vector.');
end;
[AR,AC,V]=dcv_init(v,LFilt);
if (RANDOM_INIT)
  [NStat,NSymb,LFilt,REAL_COEFF]=dcv_chk(AR,AC,V,zeros(LFilt,1));
else
  [NStat,NSymb,LFilt,REAL_COEFF]=dcv_chk(AR,AC,V,H_0);
end;

% Variables
H = zeros(LFilt, Nit+1);
sigma2 = zeros(1, Nit+1);
% Store initalization values
if (RANDOM_INIT)
  % Use arbitrary init (the values are arbitrary but one has to take care of
  % possible numerical problems with the variance)
  H(:,1) = zeros(LFilt,1);
  sigma2(1) = cov(X);
else
  H(:,1) = H_0;
  sigma2(1) = sigma2_0;
end;  
% Pre-compute the state dependent matrices (more efficient if T >> NStat)
VMAT = zeros(LFilt,LFilt*NStat);
for n = 1:NStat 
  VMAT(:,(1+(n-1)*LFilt):(n*LFilt)) = V(:,n)*V(:,n)';
end;
% Energy of the filtered signal
EX = X'*X;

% Run the Gibbs sampler using data augmentation
fprintf(1,'Iteration:');
for n = 1:Nit
  if (rem(n, NECHO) == 0)
    fprintf(1, ' %d', n);
  end;
  % Simulate a sequence of state ~ (s | X, H, sigma2)
  s = dcv_psim(X, AR, AC, V, H(:,n), sigma2(:,n), 1);
  % Computes the numerical characteristics of the regression model
  tmp = zeros(LFilt,1);
  Sigma = zeros(LFilt, LFilt);
  for l = 1:NStat
    ind = find(s == l);
    Sigma = Sigma + length(ind)* VMAT(:,(1+(l-1)*LFilt):(l*LFilt));
    tmp = tmp + sum(conj(X(ind)))*V(:,l);
  end;
  % Compute inverse explicitly (this is not very efficient but the inverse
  % is needed below and MATLAB doesn't use any specific inversion routine for
  % triangular matrices, so that it is not efficient to use chol at this point)
  Sigma = inv(Sigma);
  if (~REAL_COEFF)
    % Force the inverse of the covariance matrice to be hermitian (which it
    % is up to the numerical precision, otherwise chol won't work)
    for k = 1:LFilt
      Sigma(k,k) = real(Sigma(k,k));
    end
  end
  % NOTE (19 Nov 2004): These formulas are wrong because they pertain to the
  % least-squares estimate of H given the simulations rather than to the
  % error corresponding to H(:,n). The difference is weak but...
  MH = Sigma*tmp;
  ms2 = real(EX - MH'*tmp)/(T-LFilt);
  % Degrees of freedom of the Inv-Chi2 distribution
  if (REAL_COEFF)
    nu = T-LFilt;
  else
    nu = 2*(T-LFilt);
  end;
  % Simulate ~ (sigma2 | X, H, s)
  % Distribution: Inv-Chi2(nu, ms2) (the first solution below is
  % higly inefficient when T is large)
  %% sigma2(n+1) = nu*ms2/sum(randn(1,nu).^2);
  % Use the Inv-Gamma(nu/2, nu/2*ms2) form instead
  sigma2(n+1) = 1/randgamm(nu/2, (nu/2)*ms2);
  % Simulate ~ (H | X, sigma2, s)
  % Distribution: N (s1, S2*sigma2) (simulates using Choleski decomposition)
  F = chol(Sigma);	% Sigma = F'*F
  if (REAL_COEFF)
    H(:,n+1) = MH + sqrt(sigma2(n+1))*(F'*randn(LFilt,1));
  else
    H(:,n+1) = MH + sqrt(sigma2(n+1)) * ...
	(F'*(randn(LFilt,1)+i*randn(LFilt,1))/sqrt(2));
  end;
end;
fprintf(1,'\n');
