function [logl, gamma] = dcv_fb (X, AR, AC, V, H, sigma2);

%dcv_fb    Computes likelihood and a posteriori state probabilities using
%	   the forward-backward algorithm.
%Use:  [logl,gamma]=dcv_fb(X,AR,AC,V,H,sigma2).

% DCV Toolbox, version 1.1
% Olivier Capp, 30/12/97 - 30/12/97
% ENST Dpt. Signal / CNRS URA 820, Paris

% Input args.
error(nargchk(6, 6, nargin));
[NStat, NSymb, LFilt, REAL_COEFF] = dcv_chk(AR,AC,V,H);
[T, Nc] = size(X);
if (Nc ~= 1)
  error('Observations should be specified as a row vector.');
end;

% Compute density values
dens = zeros(T, NStat);
if (REAL_COEFF)
  for n = 1:NStat
    mu = H'*V(:,n);
    dens(:,n) = exp(-0.5*((X-mu).^2)/sigma2);
  end;
  dens = dens/sqrt(2*pi*sigma2);
else
  % Beware: In this case, sigma2 is the power of the noise which is
  % assumed to be complex and circular
  for n = 1:NStat
    mu = H'*V(:,n);
    dens(:,n) = exp(-real((X-mu).*conj(X-mu))/sigma2);
  end;
  dens = dens/(pi*sigma2);
end;

% Forward pass
scale = [1 ; zeros(T-1, 1)];
alpha = zeros(T, NStat);
% Assumes that a priori all states are equiprobables
alpha(1,:) = dens(1,:)/NStat;
TMP = zeros(NSymb, NStat);
for t=2:T
  for j=1:NSymb
    TMP(j,:) =  alpha(t-1,AC(j,:));
  end;
  % All non-null transition prob. are equal to 1/Nsymb
  alpha(t,:) = sum(TMP)/NSymb .* dens(t,:);
  % Systematic scaling
  scale(t) = sum(alpha(t,:));
  alpha(t,:) = alpha(t,:) / scale(t);
end;
% This is not computationnaly efficient but log(prod()) would produce
% underflows here
logl = log(sum(alpha(T,:))) + sum(log(scale));

% Compute a posteriori probabilities during backward pass if needed
if (nargout > 1)
  gamma = zeros(T, NStat);
  beta = ones(1, NStat);
  gamma(T,:) = alpha(T,:).*beta;
  gamma(T,:) = gamma(T,:)/sum(gamma(T,:));
  TMP = zeros(NSymb, NStat);
  for t=(T-1):-1:1
    for j=1:NSymb
      TMP(j,:) =  beta(AR(:,j)) .* dens(t+1,AR(:,j));
    end;
    % Use same scaling factor as in forward pass
    beta = (sum(TMP)/NSymb)/scale(t);
    gamma(t,:) = alpha(t,:).*beta;
    gamma(t,:) = gamma(t,:)/sum(gamma(t,:));
  end;
end;
