function [H, sigma2] = dcv_par (X, gamma, V);

%dcv_par   Re-estimates the convolution parameters (filter and noise level).
%	Use: [H,sigma2]=dcv_par(X,gamma,V) or [H,sigma2]=dcv_par(X,s,V). In
%	the first case gamma is considered as a matrix of a posteriori
%	probabilities (for the M step of the EM algorithm), in the second
%	s is a sequence of states (for input output identification, or
%	stochastic versions of the EM algorithm).

% DCV Toolbox, version 1.1
% Olivier Capp, 24/07/97 - 24/12/97
% ENST Dpt. Signal / CNRS URA 820, Paris

% Input args.
error(nargchk(3, 3, nargin));
[LFilt, NStat] = size(V);
[T, Nc] = size(X);
if (Nc ~= 1)
  error('Observations must be specified as a column vector.');
end;
[Nr, Nc] = size(gamma);
if (Nr ~= T)
  error('Second argument has an incorrect size.');
end;
if (Nc == NStat)
  PROB_FLAG = 1;
elseif (Nc == 1)
  PROB_FLAG = 0;
else
  error('Second argument has an incorrect size.');
end;

% Pre-compute the state dependent matrices (more efficient if T >> NStat)
VMAT = zeros(LFilt,LFilt*NStat);
for n = 1:NStat 
  VMAT(:,(1+(n-1)*LFilt):(n*LFilt)) = V(:,n)*V(:,n)';
end;

if (PROB_FLAG)
  % Reestimate the filter coefficients
  S2 = zeros(LFilt, LFilt);
  s1 = zeros(LFilt,1);
  gamma_s = sum(gamma);
  for n = 1:NStat
    S2 = S2 + gamma_s(n)* VMAT(:,(1+(n-1)*LFilt):(n*LFilt));
    s1 = s1 + sum(gamma(:,n).*conj(X))*V(:,n);
  end;
  H = S2\s1;

  % And the noise variance
  sigma2 = 0;
  for n = 1:NStat
    sigma2 = sigma2 ...
	+ sum(gamma(:,n) .* real((X-H'*V(:,n)) .* conj(X-H'*V(:,n))));
  end;
  sigma2 = real(X'*X - H'*s1)/T;
else
  % Reestimate the filter coefficients
  S2 = zeros(LFilt, LFilt);
  s1 = zeros(LFilt,1);
  for n = 1:NStat
    ind = find(gamma == n);
    S2 = S2 + length(ind)* VMAT(:,(1+(n-1)*LFilt):(n*LFilt));
    s1 = s1 + sum(conj(X(ind)))*V(:,n);
  end;
  H = S2\s1;
  % And the noise variance
  sigma2 = real(X'*X - H'*s1)/T;
end;
