function s = dcv_psim(X, AR, AC, V, H, sigma2, Nsim);

%dcv_psim  Simulates sequence(s) of states conditionnally to the observed data.
%Use:  s = dcv_psim(X, AR, AC, V, H, sigma2, [Nsim]).

% DCV Toolbox, version 1.1
% Olivier Capp, 23/12/97 - 24/12/97
% ENST Dpt. Signal / CNRS URA 820, Paris

% Needed functions
if (~((exist('randindx') == 2) | (exist('randindx') == 3)))
  error('Function randindx is missing.');
end;

% Input args.
error(nargchk(6, 7, nargin));
[NStat, NSymb, LFilt, REAL_COEFF] = dcv_chk(AR,AC,V,H);
[T, Nc] = size(X);
if (Nc ~= 1)
  error('Observations should be specified as a row vector.');
end;
if (nargin < 7)
  Nsim = 1;
end;

% First do the foward recursion (as in dcv_fb)
% Compute density values
dens = zeros(T, NStat);
if (REAL_COEFF)
  for n = 1:NStat
    mu = H'*V(:,n);
    dens(:,n) = 1/sqrt(2*pi*sigma2) * exp(-0.5*((X-mu).^2)/sigma2);
  end;
else
  % Beware: In this case, sigma2 is the power of the noise which is
  % assumed to be complex and circular
  for n = 1:NStat
    mu = H'*V(:,n);
    dens(:,n) = 1/(pi*sigma2) * exp(-real((X-mu).*conj(X-mu))/sigma2);
  end;
end;
% Forward pass
scale = [1 ; zeros(T-1, 1)];
alpha = zeros(T, NStat);
% Assumes that a priori all states are equiprobables
alpha(1,:) = dens(1,:)/NStat;
TMP = zeros(NSymb, NStat);
for t=2:T
  for j=1:NSymb
    TMP(j,:) =  alpha(t-1,AC(j,:));
  end;
  % All non-null transition prob. are equal to 1/Nsymb
  alpha(t,:) = sum(TMP)/NSymb .* dens(t,:);
  % Systematic scaling
  scale(t) = sum(alpha(t,:));
  alpha(t,:) = alpha(t,:) / scale(t);
end;

% Do the backward simulation
s = zeros(T,Nsim);
for n = 1:Nsim
  % Simulate the last index
  s(T,n) = randindx(alpha(T,:),1,1);
  for t=(T-1):-1:1
    % Compute probabilities conditionnal to X and s(t+1:T)
    % A common factor 1/Nsymb is missing here but it will be normalized anyway
    p = alpha(t,AC(:,s(t+1,n)));
    p = p./sum(p);
    % Conditional simulation
    s(t,n) = AC(randindx(p,1,1), s(t+1,n));
  end;
end;
