function [H, sigma2, logl] = dcv_sem(X, v, H_0, sigma2_0, Nit);

%dcv_sem   Blind identification using the SEM algorithm.
%Use:	[H,sigma2,logl] = dcv_sem(X,v,H_0,sigma2_0,Nit). Nit has to be a two
%	elements array indicating the length of the burn-in and the number of
%	subsequent iterations. Returns the estimated quantities corresponding
%	to each iteration (ie. H(:,n), sigma2(n), logl(n) for n = 1
%	(initialization) to n = Nit+1 (last iteration).

% DCV Toolbox, version 1.1
% Olivier Capp, 24/12/97 - 26/12/97
% ENST Dpt. Signal / CNRS URA 820, Paris

% Constants
NECHO = 5;

% Input args.
error(nargchk(5, 5, nargin));
% Set parameters for the DCV functions
[AR,AC,V]=dcv_init(v,length(H_0));
[NStat,NSymb,LFilt,REAL_COEFF]=dcv_chk(AR,AC,V,H_0);
if (length(Nit) ~=2)
  error('Iteration number must be a two elements array.');
end;
Nbi = Nit(1);
Nav = Nit(2);
Nit = sum(Nit);

% Estimate parameters using SEM
H = zeros(LFilt, Nit+1);
sigma2 = zeros(1, Nit+1);
% Store initalization values
H(:,1) = H_0;
sigma2(1) = sigma2_0;

fprintf(1,'Iteration:');
for n=1:Nbi
  if (rem(n, NECHO) == 0)
    fprintf(1, ' %d', n);
  end;
  % Simulate a sequence of states
  s = dcv_psim(X, AR, AC, V, H(:,n), sigma2(:,n), 1);
  % Maximize the full data likelihood imputing the simulated sequence
  [H(:,n+1),sigma2(n+1)] = dcv_par(X,s,V);
end;
if (Nav > 0)
  fprintf(1,'\nEnd of burn-in\nIteration:');
  for n=Nbi+1:Nit
    if (rem(n, NECHO) == 0)
      fprintf(1, ' %d', n);
    end;
    s = dcv_psim(X, AR, AC, V, H(:,n), sigma2(:,n), 1);
    [H(:,n+1),sigma2(n+1)] = dcv_par(X,s,V);
  end;
  % Do averaging of the parameters on the second section
  H(:,Nbi+1:Nit+1) = (cumsum(H(:,Nbi+1:Nit+1).') ./ ...
    ((1:Nit-Nbi+1).'*ones(1,LFilt))).';
  sigma2(Nbi+1:Nit+1) = cumsum(sigma2(Nbi+1:Nit+1)) ./ (1:Nit-Nbi+1);
  fprintf(1,'\n');
else
  fprintf(1,'\n');
end;

% Compute the log-likelihood if necessary
if (nargin > 2)
  fprintf(1,'Computing log-likelihoods\n');
  logl = zeros(1, Nit+1);
  for n = 1:Nit+1
    logl(n) = dcv_fb(X,AR,AC,V,H(:,n),sigma2(n));
  end;
end;
