function x = randgamm(alpha, beta);

%randgamm  Generates one deviate from the Gamma(alpha, beta) distribution.
%	Use: x = randgam(alpha, beta);

% DCV Toolbox, version 1.1
% Olivier Capp, 27/05/97 - 26/12/97
% ENST Dpt. Signal / CNRS URA 820, Paris

% References:
% [1] W. Press, S. Teukolsky, W. Vetterling, B. Flannery, "Numerical recipes
% in {C} : the art of scientific computing", Cambridge University Press,
% second edition, 1992
% [2] A. Stuart, J. K. Ord editors, "Kendall's advanced theory of statistics",
% Charles Griffin & Co, London, fifth edition, volume 1 (distribution theory),
% 1987
% [3] C. P. Robert, "Mthodes de Monte Carlo par chanes de Markov",
% Economica, 1996

% Input arguments
error(nargchk(1, 2, nargin));
if (nargin < 2)
  beta = 1;
end;

% Check that the parameters are OK
if ((alpha <= 0) | (beta <= 0))
  error('Parameters must be positive');
end;

% First take care of the case (alpha < 1)
if (alpha < 1)
  % Use Stuart theorem's ([3]-p. 26)
  alpha = 1 + alpha;
  ALPHA_SMALLER_THAN_ONE = 1;
else
  ALPHA_SMALLER_THAN_ONE = 0;
end;

% Due to MATLAB's implementation, this case is much faster than the second
% one even for large values of alpha (this is not true if the mfile is
% compiled, see also [1])
if (alpha < 200)
  m = floor(alpha);
  if (m == alpha)
    % Alpha is a small interger, sum alpha exponential deviates ([1]-p. 291)
    x = -log(prod(rand(1,m)));
  else
    % Alpha is small but not an integer, use rejection sampling with a mixture
    % of Gamma(m), Gamma(m+1) as proposal distribution ([2]-p. 307)
    q = alpha-m;
    x = 0;
    while (rand > (x/m)^q/(1+q*(x/m-1)))
      if (rand < q)
        x = -log(prod(rand(1,m+1)));
      else
        x = -log(prod(rand(1,m)));
      end;
    end;
  end;
else
  % Alpha is large, use rejection sampling with Lorentzian proposal
  % distribution ([1]-p. 292)
  alpha = alpha-1;
  % Main loop of the rejection algorithm
  e = 0;
  while (rand > e)
    % Reject x if it is negative (region of zero probability)
    x = 0;
    while (x <= 0)
      % Generate original deviate by transformation
      y = tan(pi * rand);
      x = sqrt(2*alpha+1)*y+alpha;
    end;
    % Compute probability ratio for rejection algorithm
    e = (1+y^2)*exp(alpha*log(x/alpha)-sqrt(2*alpha+1)*y);
  end;
end;

if (ALPHA_SMALLER_THAN_ONE)
  % Beware at this point (alpha <- alpha+1)
  x = x*(rand^(1/(alpha-1)));
end;

% Scale deviate
x = x/beta;
